<?php
/**
*** @name: gesa.lib.php
***
*** @description: utils functions for gesa
*** @author: Quentin George <q.george@esia-sa.com>
*** @date: 13/06/2023
**/

function getLangAvailables($langDir)
{
	$langAvailables = array();
	if(is_dir($langDir))
	{
		$dirList = array_diff(scandir($langDir), array('..', '.'));
		foreach($dirList as $element)
		{
			if(is_dir($langDir . $element))
			{
				$langAvailables[] = $element;
			}
		}
	}

	return $langAvailables;
}

function getJsonConfFile($confFile)
{
	$conf = array();
	if(file_exists($confFile))
	{
		$json = file_get_contents($confFile);
		$conf = json_decode($json, TRUE);
		if(!is_array($conf))
		{
			// json_decode return TRUE, FALSE or NULL. The file exist but cannot be opened
			throw new Exception("Cannot open conf file (" . $confFile . ")");
		}
	}

	return $conf;
}

// Save data in json file
function setJsonConfFile($confFile, $data, $isPretty=TRUE)
{
	$flag = 0;
	if($isPretty == TRUE)
	{
		$flag = JSON_PRETTY_PRINT;
	}

	$handle = fopen($confFile, "w+t");
	if(!$handle)
	{
		return FALSE;
	}
	$isWritten = fwrite($handle, json_encode($data, $flag));
	if($isWritten == FALSE)
	{
		return FALSE;
	}
	fclose($handle);

	return TRUE;
}

function getTxtConfFile($confFile, $separator='=')
{
	$conf = array();
	if(file_exists($confFile))
	{
		$content = file($confFile);
		foreach($content as $line)
		{
			$splitted = explode($separator, $line);
			$conf[trim($splitted[0])] = trim($splitted[1]);
		}
	}

	return $conf;
}

// Save data in text file
function setTxtConfFile($confFile, $data, $separator='=')
{
	// Format data as string
	$content = '';
	foreach($data as $key => $value)
	{
		$content .= trim($key) . $separator . trim($value) . "\n";
	}

	// Open file then write content
	$handle = fopen($confFile, "w+t");
	if(!$handle)
	{
		return FALSE;
	}
	$isWritten = fwrite($handle, $content);
	if($isWritten == FALSE)
	{
		return FALSE;
	}
	fclose($handle);

	return TRUE;
}

function netmaskFromCidr($cidr)
{
	$binArr = str_split(str_pad(str_pad('', $cidr, '1'), 32, '0'), 8);
	$decArr = array();
	foreach($binArr as $value)
	{
		$decArr[] = bindec($value);
	}

	return implode('.', $decArr);
}

function handleExecError($code, $output=array())
{
	if($code > 0)
	{
		$msg = 'Command failed with error code: ' . $code . '.';
		if(!empty($output))
		{
			$msg .= ' Message: <br>' . implode('<br>', $output);
		}

		throw new Exception($msg);
	}
}

function getFileLastLine($file)
{
	$line = '';
	$cursor = 0;
	$handle = fopen(GESA_LOGS_FILE, 'r');
	// Trim trailing newline characters in the file
	do
	{
		fseek($handle, $cursor--, SEEK_END);
		$char = fgetc($handle);
	}
	while($char == "\n" || $char == "\r" || $char == "");
	// Read until the next line of the file begins or the first newline char
	while($char !== FALSE && $char != "\n" && $char != "\r")
	{
		// Prepend the new character
		$line = $char . $line;
		fseek($handle, $cursor--, SEEK_END);
		$char = fgetc($handle);
	}
	fclose($handle);

	return trim($line);
}

function getGesaLastUpdate($infoFile=GESA_UPDATES_INFO_FILE)
{
	// Get last update infos
	$updates = array(
        'last_start' => '',
        'last_end' => '',
    );
    if(file_exists($infoFile))
    {
        $content = file_get_contents($infoFile);
        if(preg_match_all('/[0-9].+/', trim($content), $ret))
        {
            if(!empty($ret[0][0]))
            {
                $updates['last_start'] = $ret[0][0];
            }
            if(!empty($ret[0][1]))
            {
                $updates['last_end'] = $ret[0][1];
            }
        }
    }

	return $updates;
}

function getIfsInfos($isLO=FALSE, $precision=2)
{
	// Get interfaces infos
	$interfaces = array();
    $ifsArray = shell_exec('ip -a -s -j addr');
	if(!empty($ifsArray))
	{
		$ifsArray = json_decode($ifsArray, TRUE);
        // If $isLO == FALSE, start at $i=1 because $ifsArray[0] is the loopback and we don't need it
        for($isLO ? $i=0 : $i=1; $i < count($ifsArray); $i++)
        {
            $iface = array(
                'index' => $ifsArray[$i]['ifindex'],
                'name' => $ifsArray[$i]['ifname'],
                'state' => strtolower($ifsArray[$i]['operstate']),
				'speed' => Convert\bitsPerSecond(($ifsArray[$i]['txqlen'] * 1000000), $precision), // For convertion libs to work transform this value from mbps to bps
                'mtu' => $ifsArray[$i]['mtu'],
                'ip' => $ifsArray[$i]['addr_info'][0]['local'],
                'in_octet' => Convert\bytes($ifsArray[$i]['stats64']['rx']['bytes'], $precision),
                'out_octet' => Convert\bytes($ifsArray[$i]['stats64']['tx']['bytes'], $precision),
                'in_error' => $ifsArray[$i]['stats64']['rx']['errors'],
                'out_error' => $ifsArray[$i]['stats64']['tx']['errors'],
            );

			// Push raw data for speeds sort
			$iface['speed']['raw'] = $ifsArray[$i]['txqlen'];
			$iface['in_octet']['raw'] = $ifsArray[$i]['stats64']['rx']['bytes'];
			$iface['out_octet']['raw'] = $ifsArray[$i]['stats64']['tx']['bytes'];
			$iface['in_error']['raw'] = $ifsArray[$i]['stats64']['rx']['errors'];
			$iface['out_error']['raw'] = $ifsArray[$i]['stats64']['tx']['errors'];

			$interfaces[] = $iface;
        }
	}

	return $interfaces;
}

// Check if a string ($str) ends with case sensitive ($strEnd)
function strEndsWith($str, $strEnd)
{
	if(substr_compare($str, $strEnd, -strlen($strEnd)) === 0)
    {
        return TRUE;
    }

	return FALSE;
}

// Get list of files by file name end (can be an extension)
function getFilesListByNameEnd($pathDir, $protExt, $excludeList=array())
{
	$ret = array();
    $filesList = @scandir($pathDir);
    foreach($filesList as $file)
    {
        if(strEndsWith($file, $protExt) && !in_array($file, $excludeList))
        {
            $ret[] = $file;
        }
    }

    return $ret;
}

// Get a key value from ecatp.conf file
function getEcatpConfFileValue($confKey)
{
	// $confKey is configure in gesa.conf.php ECATP_CONF constant
	$value = '';
	// Default value
	if(isset(ECATP_CONF[$confKey]['DEFAULT']))
	{
		$value = ECATP_CONF[$confKey]['DEFAULT'];
	}
	// Get file content
	if(file_exists(ECATP_CONF['PATH']))
	{
		$content = parse_ini_file(ECATP_CONF['PATH']);
		if(isset($content[ECATP_CONF[$confKey]['KEY']]))
		{
			$value = $content[ECATP_CONF[$confKey]['KEY']];
			$value = trim($value);
		}
	}

	return $value;
}

// Get availlable interfaces from the OS (no loopback and no docker)
function getOSValidIfaces()
{
	$interfacesList = array();
	$ifsArray = shell_exec('ip -a -j addr');
	if(!empty($ifsArray))
	{
		$ifsArray = json_decode($ifsArray, TRUE);
		foreach($ifsArray as $iface)
		{
			if($iface['link_type'] != 'loopback' && !preg_match('/docker/i', $iface['ifname']))
			{
				$interfacesList[] = $iface;
			}
		}
	}

	return $interfacesList;
}

?>
